import { AsyncAbstractDriverFactory, SyncAbstractDriverFactory, AsyncAbstractDriverInterface, SyncAbstractDriverInterface, SyncModels } from '..';
import { KeyValueModel as SyncKeyValueModel } from '../models/sync/key-value-model.class';
import { CollectionModel as SyncCollectionModel } from '../models/sync/collection-model.class';
import { KeyValueModel as AsyncKeyValueModel } from '../models/async/key-value-model.class';
import { CollectionModel as AsyncCollectionModel } from '../models/async/collection-model.class';
export declare type PushDataItem = {
    type: 'put' | 'forget';
    /**
     * This pushed data belongs to the model name.
     */
    modelName: string;
    /**
     * The millisecond timestamp when this data was changed.
     */
    timestampMs: number;
    /**
     * The key of the data in the model to which it belongs.
     */
    key: string;
    /**
     * The value of this data, this value is always null because the intent is to delete.
     */
    value: unknown;
};
export declare type PushData = Array<PushDataItem>;
export declare type WaitingPushDataItem = PushDataItem & {
    /**
     * This value represents whether the data is being processed by a pushdatahandler, similar to a lock.
     */
    pushing: boolean;
};
export declare type WaitingPushData = Array<WaitingPushDataItem>;
export interface BaseCloudSynchronizerOptionsInterface {
    /**
     * default: false
     */
    disabled?: boolean;
    /**
     * Synchronize these models.
     */
    models: Array<SyncCollectionModel | SyncKeyValueModel | AsyncCollectionModel | AsyncKeyValueModel>;
    /**
     * Whether to enable automatic push.
     * If true, when the data in the model is changed,
     * if no new data is changed within a certain period of time (determined by "minSyncInterval"),
     * push will be automatically executed.
     *
     * default: true
     */
    autoPush?: boolean;
    /**
     * Minimum model data synchronization interval (ms)
     * When the data is updated, Kurimudb will debounce, to avoid sending too many requests to the server at the same time.
     * This value is the milliseconds of debounces performed.
     * Please determine the specific value according to the tolerable delay in your business.
     *
     * default: 2000
     */
    minSyncInterval?: number;
    /**
     * How often do I retry when the operation fails (ms)
     *
     * default: 1000
     */
    retryDelayInterval?: number;
    /**
     * Maximum number of retries.
     *
     * default: 3
     */
    maxRetries?: number;
    /**
     * Which driver do you want to use to store the cached data generated during synchronization.
     * It's best to choose the driver your model is using.
     */
    driver: SyncAbstractDriverFactory | AsyncAbstractDriverFactory;
    /**
     *
     */
    pushDataHandler: (data: PushData) => Promise<boolean> | boolean;
    /**
     *
     */
    pullDataHandler: (lastPullTimestampMs: number) => Promise<PushData> | PushData;
}
export interface BaseCloudSynchronizerInterface {
    syncStatus: 'pending' | 'succeed' | 'defeated';
    _waitingPushData: WaitingPushData;
    _lastPullTimestampMs: number;
    [_keyUpdateTimestampMs: string]: any;
}
export declare class BaseCloudSynchronizer extends SyncModels.keyValue<BaseCloudSynchronizerInterface> {
    readonly cloudSyncOptions: BaseCloudSynchronizerOptionsInterface;
    readonly cloudSyncStorage: AsyncAbstractDriverInterface | SyncAbstractDriverInterface;
    readonly models: import("..").KMap<SyncCollectionModel<any, undefined> | SyncKeyValueModel<Record<string, any>, undefined> | AsyncCollectionModel<any, AsyncAbstractDriverInterface> | AsyncKeyValueModel<Record<string, any>, AsyncAbstractDriverInterface>>;
    constructor(cloudSyncOptions: BaseCloudSynchronizerOptionsInterface);
    pull(): Promise<void>;
    pushDebounceTimer: number | 'busy' | 'idle';
    push(passive?: boolean): void;
    private _pushData;
    private _recursionPush;
    private _pushWaitingPushData;
    private firstSync;
    private _processWaitingPushData;
    private _processedWaitingPushData;
    storageGetItem<Key extends keyof BaseCloudSynchronizerInterface>(key: Key): Promise<BaseCloudSynchronizerInterface[Key] | undefined>;
    storageSetItem<Key extends keyof BaseCloudSynchronizerInterface>(key: Key, value: BaseCloudSynchronizerInterface[Key]): Promise<void>;
}
