var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { cacheFactory, runtime, syncDataFactory } from '../../providers';
import { makeKMap } from '../../helpers/make-kurimudb-map.func';
import { clone } from '../../helpers/clone.func';
import { isPromise } from '../../helpers/is-promise.func';
let auto$nextId = 0;
export class BaseModel {
    constructor(options) {
        this._curdSubscribeId = 1;
        this._curdSubscribeFuncs = {
            setItem: {},
            removeItem: {},
        };
        this.options = this.checkOptions(options);
        this.cache = cacheFactory.make(this);
        this.$ = this.cache.changed.subscribe;
        this.data = syncDataFactory.make(this);
        if (undefined !== options.driver) {
            this.storage = options.driver.make(this);
        }
        else {
            this.storage = undefined;
        }
    }
    checkOptions(options) {
        var _a, _b;
        if (!options.name)
            throw new Error(`[Kurimudb] The model "name" does not exist.`);
        if (!options.ioType || !options.modelType) {
            throw new Error(`[Kurimudb] The model "ioType" or "modelType" does not exist. Do you inherit "CollectionModel" or "KeyValueModel"? Please don't do that.`);
        }
        options.primary = (_a = options.primary) !== null && _a !== void 0 ? _a : '_id';
        options.async = (_b = options.async) !== null && _b !== void 0 ? _b : false;
        return options;
    }
    getItem(key) {
        const skey = String(key);
        const result = this.cache.get(skey);
        if (undefined !== result)
            return result;
        if (undefined === this.storage)
            return undefined;
        const value = this.storage.select(skey);
        this.cache.put(skey, value);
        return value;
    }
    bulkGetItem(keys) {
        if (undefined === this.storage) {
            const results = makeKMap();
            for (const key of keys) {
                const skey = String(key);
                results[skey] = this.cache.get(skey);
            }
            return results;
        }
        else {
            const results = makeKMap();
            const storageQueryItems = [];
            for (const key of keys) {
                const skey = String(key);
                results[skey] = this.cache.get(skey);
                if (undefined === results[skey])
                    storageQueryItems.push(skey);
            }
            const data = this.storage.bulkSelect(storageQueryItems);
            for (const skey in data) {
                results[skey] = data[skey];
                this.cache.put(skey, data[skey]);
            }
            return results;
        }
    }
    setItem(key, value) {
        value = this.clone(value);
        const skey = String(key);
        this.cache.publish(skey, value);
        if (undefined === this.storage)
            return;
        this.storage.insertOrUpdate(skey, value);
        // trigger subscriptions
        for (const id in this._curdSubscribeFuncs.setItem) {
            const closFunc = this._curdSubscribeFuncs.setItem[id];
            closFunc(value, skey);
        }
    }
    bulkSetItem(items) {
        items = this.clone(items);
        const skeyItems = makeKMap();
        for (const key in items)
            skeyItems[String(key)] = items[key];
        if (undefined !== this.storage) {
            if (false === this.storage.bulkInsertOrUpdate(skeyItems))
                return false;
        }
        for (const skey in skeyItems) {
            this.cache.publish(skey, skeyItems[skey]);
        }
        // trigger subscriptions
        for (const id in this._curdSubscribeFuncs.setItem) {
            const closFunc = this._curdSubscribeFuncs.setItem[id];
            skeyItems.forEach((value, key) => {
                closFunc(value, key);
            });
        }
        return true;
    }
    setItem$(closFunc) {
        const id = ++this._curdSubscribeId;
        this._curdSubscribeFuncs.setItem[id] = closFunc;
        return () => delete this._curdSubscribeFuncs.setItem[id];
    }
    hasItem(key) {
        const skey = String(key);
        if (this.cache.has(skey))
            return true;
        if (undefined === this.storage)
            return false;
        return this.storage.exists(skey);
    }
    removeItem(key) {
        const skey = String(key);
        this.cache.forget(skey);
        // trigger subscriptions
        for (const id in this._curdSubscribeFuncs.removeItem) {
            const closFunc = this._curdSubscribeFuncs.removeItem[id];
            closFunc(skey);
        }
        if (undefined === this.storage)
            return;
        this.storage.delete(skey);
    }
    bulkRemoveItem(keys) {
        const skeys = keys.map((k) => String(k));
        if (undefined !== this.storage) {
            if (false === this.storage.bulkDelete(skeys))
                return false;
        }
        for (const key of keys)
            this.cache.forget(String(key));
        // trigger subscriptions
        for (const id in this._curdSubscribeFuncs.removeItem) {
            const closFunc = this._curdSubscribeFuncs.removeItem[id];
            skeys.forEach((key) => {
                closFunc(key);
            });
        }
        return true;
    }
    removeItem$(closFunc) {
        const id = ++this._curdSubscribeId;
        this._curdSubscribeFuncs.removeItem[id] = closFunc;
        return () => delete this._curdSubscribeFuncs.removeItem[id];
    }
    subscribeItem(key, closFunc, config) {
        return this.getSubscribeItemFunc(key)(closFunc, config);
    }
    getSubscribeItemFunc(key) {
        const skey = String(key);
        return this.cache.subscribe(skey);
    }
    clone(value) {
        if (undefined === this.storage) {
            return clone(value);
        }
        else {
            return this.storage.clone(value);
        }
    }
    auto$(subClosFunc, config = {}) {
        config.immediate = false; // 所有用 auto$ 进行的订阅，均不主动触发首次订阅，由队列函数去主动触发
        if (undefined === runtime.readModelInternalItemDependencies[this.options.name]) {
            runtime.readModelInternalItemDependencies[this.options.name] = {};
        }
        const id = auto$nextId++;
        runtime.readModelInternalItemDependencies[this.options.name][id] = [];
        const r = subClosFunc();
        const subscribe = () => {
            const unsubscribeFuncArr = runtime.readModelInternalItemDependencies[this.options.name][id].map((item) => {
                return item.subscribe(subClosFunc, config);
            });
            delete runtime.readModelInternalItemDependencies[this.options.name][id];
            return () => {
                unsubscribeFuncArr.forEach((unsubscribe) => unsubscribe());
            };
        };
        if (!isPromise(r)) {
            return subscribe();
        }
        else {
            return (() => __awaiter(this, void 0, void 0, function* () {
                yield r;
                return subscribe();
            }))();
        }
    }
    stream$(getData, handler, config = {}) {
        var _a;
        config.immediate = false; // 所有用 auto$ 进行的订阅，均不主动触发首次订阅，由队列函数去主动触发
        config.autoUnsubscribe = false; // 因为会动态订阅新的值，主流框架不支持，因此不执行自动退订功能
        let unsubscribing = false;
        const subscribing = new Map();
        let isAutoLoad = (_a = config === null || config === void 0 ? void 0 : config.autoLoad) !== null && _a !== void 0 ? _a : false;
        let unsubscribeModel = undefined;
        const load = () => __awaiter(this, void 0, void 0, function* () {
            const data = (yield getData());
            let isSubscribeModel = false;
            if (Array.isArray(data)) {
                if (isAutoLoad && 0 === data.length)
                    isSubscribeModel = true;
                const keys = data.filter((k) => !subscribing.has(k));
                const results = yield this.bulkGetItem(keys);
                keys.forEach((k) => {
                    if (unsubscribing)
                        return;
                    subscribing.set(k, this.subscribeItem(k, (v) => handler(k, v), config));
                });
                for (const k in results)
                    handler(k, results[k]);
            }
            else {
                if (isAutoLoad)
                    isSubscribeModel = true;
                for (const key in data) {
                    if (isAutoLoad)
                        isSubscribeModel = false;
                    const k = key;
                    if (unsubscribing)
                        return;
                    if (subscribing.has(k))
                        continue;
                    subscribing.set(k, this.subscribeItem(k, (v) => handler(k, v), config));
                    handler(k, data[k]);
                }
            }
            if (isAutoLoad) {
                if (true === isSubscribeModel) {
                    unsubscribeModel = this.$(load, {
                        autoUnsubscribe: false,
                        immediate: false,
                    });
                }
                else {
                    if (undefined !== unsubscribeModel)
                        unsubscribeModel();
                }
            }
        });
        load();
        return {
            load: load,
            startAutoLoad: () => {
                isAutoLoad = true;
                load();
            },
            stopAutoLoad: () => {
                isAutoLoad = false;
            },
            unsubscribe: () => {
                unsubscribing = true;
                if (undefined !== unsubscribeModel)
                    unsubscribeModel();
                subscribing.forEach((uf) => uf());
            },
            getKeys: () => subscribing.keys(),
            getLength: () => subscribing.size,
        };
    }
}
//# sourceMappingURL=base-model.class.js.map